<?php

namespace Litepie\Slider\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Slider\Events\SliderAction as SliderActionEvent;
use Litepie\Slider\Notifications\SliderAction as SliderActionNotification;
use Litepie\Actions\Concerns\AsAction;

class SliderAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the SliderActionEvent.
     *
     * @param   SliderActionEvent  $event
     * @return mixed
     */
    public function handle(SliderActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $slider.
     *
     * @param   SliderActionEvent  $event
     * @return void
     */
    public function create(SliderActionEvent $event)
    {
        $client = $event->$slider->client;
        Notification::send($client, new SliderActionNotification($event));
    }

    /**
     * Handle the SliderActionEvent as a listener.
     *
     * @param   SliderActionEvent  $event
     * @return mixed
     */
    public function asListener(SliderActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   SliderActionEvent  $event
     * @return bool
     */
    private function isAllowed(SliderActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
