<?php

namespace Litepie\Slider\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Slider\Events\SliderWorkflow as SliderWorkflowEvent;
use Litepie\Slider\Notifications\SliderWorkflow as SliderWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class SliderWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(SliderWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $slider is submitted.
     *
     * @param SliderWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(SliderWorkflowEvent $event)
    {
        $client = $event->$slider->client;
        Notification::send($client, new SliderWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $slider is published.
     *
     * @param SliderWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(SliderWorkflowEvent $event)
    {
        $client = $event->$slider->client;
        Notification::send($client, new SliderWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $slider is approved.
     *
     * @param SliderWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(SliderWorkflowEvent $event)
    {
        $client = $event->$slider->client;
        Notification::send($client, new SliderWorkflowNotification($event));
    }

    /**
     * Handles the $slider workflow event as a listener.
     *
     * @param SliderWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(SliderWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
